#include "Main.h"



int main(void)
{
   int status = 0 ;

   // Declare environnement and linear problem.
   CPXENVptr env ;
   CPXLPptr lp ;
   
   // Environnement Cplex
   env = CPXopenCPLEX(&status) ;
   CPXsetintparam(env, CPX_PARAM_SCRIND, CPX_ON) ;

   //----
   // Create problem
   char ProbName[TAILLESTR] ;
   strcpy(ProbName, "Simple") ;
   lp = CPXcreateprob(env, &status, ProbName) ;

   // Define objectif function type
   CPXchgobjsen(env, lp, CPX_MAX) ;
   // Define Prob type : LP, MILP,...
   CPXchgprobtype(env, lp, CPXPROB_MILP) ;
   
   // Number of new constraints
   int rcnt = 2 ;
   // Right Hand constraint value
   double rhs[TAILLEROW] ;
   rhs[0] = 20 ;
   rhs[1] = 30 ;
   // Sense of each constrain.
   char sense[TAILLEROW] ;
   sense[0] = 'L' ;
   sense[1] = 'L' ;
   // Range of contraint if sense is a range ('R')
   double rngval[TAILLEROW] ;
   rngval[0] = 0 ;
   rngval[1] = 0 ;

   // Name of contraints
   char ** rowname ;
   rowname = new char*[TAILLEROW] ;

   for (uint16 i = 0 ; i<TAILLEROW ; i++)
   {
      rowname[i] = new char[TAILLESTR] ;
   }
   strcpy(rowname[0], "C1") ;
   strcpy(rowname[1], "C2") ;

   status = CPXnewrows(env, lp, rcnt, rhs, sense, rngval, rowname) ;
  

   // Number of column
   int ccnt = 3 ;
   // Number of nonzero constraint coefficient
   int nzcnt = 6 ;
   // Coef objective
   double obj[TAILLECOL] ;
   obj[0] = 1 ;
   obj[1] = 2 ;
   obj[2] = 3 ;
   // Array that specifies the nonzero elements of the columns being added
   int cmatbeg[TAILLECOL] ;
   cmatbeg[0] = 0 ;
   cmatbeg[1] = 2 ;
   cmatbeg[2] = 4 ;
   // Array that specifies the nonzero elements of the columns being added
   int cmatind[TAILLE] ;
   cmatind[0] = 0 ;
   cmatind[1] = 1 ;
   cmatind[2] = 0 ;
   cmatind[3] = 1 ;
   cmatind[4] = 0 ;
   cmatind[5] = 1 ;
   // Coeficient constrain
   double cmatval[TAILLE] ;
   cmatval[0] = -1.0 ;
   cmatval[1] = 1.0 ;
   cmatval[2] = 1.0 ;
   cmatval[3] = -3.0 ;
   cmatval[4] = 1.0 ;
   cmatval[5] = 1.0 ;
   // Lower bound
   double lb[TAILLECOL] ;
   lb[0] = 0.0 ;
   lb[1] = 0.0 ;
   lb[2] = 0.0 ;
   // Upper bound
   double ub[TAILLECOL] ;
   ub[0] = 40 ;
   ub[1] = CPX_INFBOUND ;
   ub[2] = CPX_INFBOUND ;
   // Column name
   char ** colname ;
   colname = new char*[TAILLECOL] ;
   for (uint16 i = 0 ; i<TAILLECOL ; i++)
   {
      colname[i] = new char[TAILLESTR] ;
   }
   strcpy(colname[0], "x1") ;
   strcpy(colname[1], "x2") ;
   strcpy(colname[2], "x3") ;

   status = CPXaddcols(env, lp, ccnt, nzcnt, obj, cmatbeg, cmatind, cmatval, lb, ub, colname) ;

   // Define variable type
   int indice[TAILLECOL] ;
   indice[0] = 0 ;
   indice[1] = 1 ;
   indice[2] = 2 ;
   char xctype[TAILLECOL] ;
   xctype[0] = 'I' ;
   xctype[1] = 'I' ;
   xctype[2] = 'I' ;
   CPXchgctype(env, lp, 3, indice, xctype) ;

   // Write output file .lp
   char filename_str[TAILLESTR] ;
   strcpy(filename_str, "uuu.lp") ;
   CPXwriteprob(env, lp, filename_str, NULL) ;



   //----
   // Resolution
   // Nombre de colonnes
   double objval = 0.0 ;
   uint16 NbCol = 0 ;
   NbCol = CPXgetnumcols (env, lp) ;

   //CPXlpopt(env, lp) ;
   CPXmipopt(env, lp) ;

   CPXgetstat(env, lp) ;
   
   //CPXgetobjval(env, lp, &objval) ;
   CPXgetmipobjval(env, lp, &objval) ;

   printf("Objectif value = %5.2f\n", objval) ;   
   
   double VariableValue[TAILLE] ;
   CPXgetx(env, lp, VariableValue, 0, NbCol - 1) ;

   char ** VariableNameTAB ;
   VariableNameTAB = new char*[NbCol] ;
   char VariableName[TAILLESTR] ;

   int Surplus = 0 ;
   CPXgetcolname(env, lp, VariableNameTAB, VariableName, TAILLESTR, &Surplus, 0, NbCol - 1) ;

   for (uint16 i = 0 ; i<NbCol ; i++)
   {
      printf("%s = %5.2f\n", VariableNameTAB[i], VariableValue[i]) ;      
   }


   //-----
   // Free space alloc  
   for (uint16 i = 0 ; i<TAILLECOL ; i++)
   {
      delete rowname[i] ;
   }
   delete rowname ;
   for (uint16 i = 0 ; i<TAILLECOL ; i++)
   {
      delete colname[i] ;
   }
   delete colname ;

   //delete VariableNameTAB ;
   CPXfreeprob(env, &lp) ;
   CPXcloseCPLEX(&env) ;

   return 0 ;
}


